namespace eval ::Waif {
	global WaifGui
	namespace export waif_play waif_pause waif_next \
		waif_prev waif_show waif_shuffle waif_stop \
		waif_vol waif_wvol waif_add waif_del waif_jump waif_insert \
		waif_save waif_reshuffle waif_clear waif_playonce waif_search \
		waif_create waif_version waif_help
	namespace import ::Waif::Playlist::playlist_read
	namespace import ::Waif::Playlist::playlist_write
	namespace import ::Waif::Playlist::playlist_shuffle
	namespace import ::Waif::Playlist::playlist_add
	namespace import ::Waif::Playlist::playlist_del
	namespace import ::Waif::Playlist::playlist_insert
	namespace import ::Waif::Mime::mime_get
	namespace import ::Waif::Mime::mime_select
	namespace import ::Waif::Playlist::playlist_search

proc waif_write { file value } {
	set procName waif_write 
	if [catch { open $file w } f] {
		log_error "<$file,$value>: $f"
		return
	}
	puts $f $value
	close $f
}

proc waif_read { file } {
	set procName waif_read
	if [catch { open $file r } f] {
		log_error "<$file>: $f"
		return
	}
	gets $f line
	close $f
	return $line
}

proc waif_which { app {debug 0} } { 
	global env
	if $debug { puts "looking for $app :" }
	foreach dir [split $env(PATH) ":"] {
		if $debug { puts -nonewline "checking $dir... " ; flush stdout }
		if [file exists $dir/$app] {
		if $debug { puts "yes" }
			return $dir/$app
		} else {
			if $debug { puts "no" }
		}
	}
	return
}

proc waif_playonce { {playfile ~/.waif/playqueue} } {
	global waifInfo
	set waifInfo(playonce) 1
	waif_play $playfile
}

proc waif_play { {playfile ~/.waif/playqueue} } {
	global waifInfo

	# buffering
	set buff_delay [Waif::Configure::configure_get buff_delay]
	if { "$buff_delay" == "" } { 
		set buff_delay [Waif::Configure::configure_set buff_delay 400]
	}

	if [file exists ~/.waif/.waif.pid] {
		waif_stop
		after 2000
	}
	set pid [fork]
	if $pid==0 {
		set waif_pid [pid]
		waif_write ~/.waif/.waif.pid $waif_pid
		set playlist [playlist_read $playfile]
		playlist_write ~/.waif/playqueue $playlist
		foreach file [list ~/.waif/current ~/.waif/next] {
			if { (![file exists $file]) || ("$playfile" != "~/.waif/playqueue") } {
				waif_write $file 0
			}
		}
		while { [file exists ~/.waif/current] } {
			set current [waif_read ~/.waif/next]
			set playlist [playlist_read ~/.waif/playqueue]
			if ![llength $playlist] {
				puts "\[waif_play] no files in playqueue: bye!"
				exit 1
			}
			set current [expr $current % [llength $playlist]]
			set next [expr $current + 1]
			waif_write ~/.waif/current $current
			waif_write ~/.waif/next $next
			set mp3 [lindex $playlist $current]
			if { ([file exists $mp3]) || ([regexp ^(http|rtsp):// $mp3]) || \
				([regexp ^cda:// $mp3]) } {
				# pause for 1/2 second - necessary for jump w/ buffering
				after $buff_delay
				set cmd [mime_select $mp3]
				set mpg123_pid [eval spawn -noecho $cmd]
				waif_write ~/.waif/.mpg123.pid $mpg123_pid
				set timeout -1
				expect eof { 
					wait -i $spawn_id
					set next [waif_read ~/.waif/current]
					incr next
					waif_write ~/.waif/current $next
				}
				if { ([info exists waifInfo(playonce)]) && $waifInfo(playonce) } {
					break
				}
			}
		}
		catch { file delete -force ~/.waif/.waif.pid }
		catch { file delete -force ~/.waif/.mpg123.pid }
	} 
		
}

proc waif_stop { } {
	set procName waif_stop
	set waif_pid [waif_read ~/.waif/.waif.pid]	
	# fix for a problem with .waif.pid being left over but no .mpg123.pid
	# previous fix was rm -r ~/.waif*  - i think this one is much nicer :)
	# we don't need to worry about ~/.waif.pid b/c that's how we get here in 
	# the first place
	if [file exists ~/.waif/.mpg123.pid] {
		set mpg123_pid [waif_read ~/.waif/.mpg123.pid]
		catch { exec kill -TERM $mpg123_pid } 
	}
	if [catch { exec kill -TERM $waif_pid } err] {
		log_error "$err"
	} 
	waif_write ~/.waif/next [waif_read ~/.waif/current]
	catch { file delete -force ~/.waif/.waif.pid }
	catch { file delete -force ~/.waif/.mpg123.pid }
}

proc waif_pause { } {

	set mpg123_pid [waif_read ~/.waif/.mpg123.pid]
	if { "$mpg123_pid" == "" } { return }
	set waif_pause_flag [waif_read ~/.waif/paused]
	if { "$waif_pause_flag" != "" } {
		catch { exec kill -CONT $mpg123_pid }
		file delete -force ~/.waif/paused
	} else {
		catch { exec kill -TSTP $mpg123_pid }
		after 500
		catch { exec kill -STOP $mpg123_pid }
		waif_write ~/.waif/paused 1
	}
}	

proc waif_next { {num 1} } {

	set procName waif_next
	set mpg123_pid [waif_read ~/.waif/.mpg123.pid]
	set next [waif_read ~/.waif/next]
	set next [expr $next + ($num - 1)]
	waif_write ~/.waif/next $next
	if [catch { exec kill -TERM $mpg123_pid } err] {
		log_error "$err"
	}
}

proc waif_prev { {num 1} } {

	set procName waif_prev
	set mpg123_pid [waif_read ~/.waif/.mpg123.pid]
	set next [waif_read ~/.waif/next]
	incr next [expr -$num - 1]
	waif_write ~/.waif/next $next
	if [catch { exec kill -TERM $mpg123_pid } err] {
		log_error "$err"
	}
}

proc waif_jump { num } {
	set procName waif_jump
	set mpg123_pid [waif_read ~/.waif/.mpg123.pid]
	# because play will increment current to obtain next
	waif_write ~/.waif/current [expr $num - 1]
	waif_write ~/.waif/next $num
	if [catch { exec kill -TERM $mpg123_pid } err] {
		log_error "$err"
	}
}

proc waif_show { {num %d} } {
	set procName waif_show
	if { "$num" == "%d" } {
		# get the defaultshow
		set num [Waif::Configure::configure_get defaultshow]
	}
	set playlist [playlist_read ~/.waif/playqueue]
	#puts "playlist = $playlist"
	set length [llength $playlist]
	if !$length return
	set idx [waif_read ~/.waif/current]
	set idx [expr $idx % $length]
	if { ("$num" == "current") || (![string compare "$num" "+0"]) } {
		puts [waif_show_file $idx $length [lindex $playlist $idx]]
		return
	} elseif { "$num" == "all" } {
		set num [expr $length - 1]
		set num +$num
	} 
	puts "* Now Playing *"
	puts [waif_show_file $idx $length [lindex $playlist $idx]]
	# is the number relative or absolute?
	if { [regexp -- "^(\[+\-])(\[0-9]\[0-9]*)$" $num all drx jdx] } {
		# this is relative
		#puts "relative range: all = $all, drx = $drx, jdx = $jdx"
		puts "* show format : $all *"
		if { "$drx" == "+" } { 
			set incr 1
			set p [expr ($idx + 1) % $length]
		} else {
			set incr -1
			set p [expr ($idx - 1) % $length]
		}
		# this is the tricky part, k will count how many steps
		# through the list insuring we get jdx number of
		# indices while p is our index place, incr determines
		# whether we go backwards or forwards
		for { set k 0 } { $k < $jdx } { incr k } {
			# keep this bound to the right length
			lappend idxlist [expr $p % $length]
			set p [expr $p % $length]
			puts [waif_show_file $p $length [lindex $playlist $p]]
			incr p $incr
		}
		#puts "idxlist = $idxlist"
	} elseif { [regexp -- "^(\[0-9]\[0-9]*)-(\[0-9]\[0-9]*)" $num all a z] } {
		# absolute range
		#puts "absolute range: all = $all, a = $a, z = $z"
		puts "* show format : $a - $z *"
		set p $idx
		if { $z >= $length } {
			set z [expr $length - 1]
		}
		# p is our place index, a is the first index and z
		# is the last, pick up all indices between a and z
		for { set p $a } { $p <= $z } { incr p } {
			puts [waif_show_file $p $length [lindex $playlist $p]]
			#lappend idxlist $p
		}
		#puts "idxlist = $idxlist"
	} elseif { [regexp -- "^(\[0-9]\[0-9]*)" $num all jdx] } {
		# absolute
		#puts "absolute : all = $all, jdx = $jdx"
		puts "* show format : $jdx *"
		if { $jdx >= $length } {
			set idx [expr $jdx - 1]
		} else {
			set idx $jdx
		}
		puts [waif_show_file $jdx $length [lindex $playlist $jdx]]
	} else {
		error "unrecognized show format: $num"
	}	
}

proc waif_show_file { idx length file } {
	global WaifGui
	set procName waif_show_file
	# this proc is a wrapper to put a file in the proper format for
	# displaying via show

	# pretty-format based on the number of files in the playlist
	set dLen [string length $length]
	if [regexp ^(http|rtsp):// $file] {
		set track $file
	} elseif { [regexp ^cda:// $file] } {
		set track [file tail $file]
		set trackinfo [file split $file]
		set artist [lindex $trackinfo 1]
		set title [lindex $trackinfo 2]
		if { ("$artist" != "unknown") && ("$title" != "") } {
			set track \
			[format "(CD Track %2d) $artist - $title" $track]
			#set track "(CD Track $track) $artist - $title"
		} else {
			set track "CD Track $track"
		}
		#return $track
	} else {
		set suffix [string tolower [file extension $file]]
		regsub ^\. $suffix "" suffix
		if { ("$suffix" == "mp3") || ("$suffix" == "ogg") } {
			catch { show_$suffix fileinfo $file }
			if { ([info exists fileinfo(artist)]) && \
				([info exists fileinfo(songname)])} {
				set track "$fileinfo(artist) - $fileinfo(songname)"
			} else {
				set track [file tail $file]
			}
		} else {
			set track [file tail $file]
		}
				
	}
	if { [info exists WaifGui] } {
		#puts "welcome to gui hooks"
		lappend WaifGui(showtracks) $track
		#puts "appending to WaifGui(showtracks) : $track"
	} else {
		#return "\[$idx] $track"
		return [format "\[%[subst $dLen]d] $track" $idx]
	}
}

proc waif_shuffle { } {
	
	set procName waif_shuffle
	set mpg123_pid [waif_read ~/.waif/.mpg123.pid]
	set waif_pid [waif_read ~/.waif/.waif.pid]
    if [file exists ~/.waif/unshuffle.$waif_pid] {
        file rename -force ~/.waif/unshuffle.$waif_pid ~/.waif/playqueue
    } else {
    	# clean previous unshuffles which are not cleaned on waif_stop
		catch {
			foreach unshuffle [glob ~/.waif/unshuffle.*] {
    			file delete -force $unshuffle
			}
		}
    	set playlist [playlist_read ~/.waif/playqueue]
    	playlist_write ~/.waif/unshuffle.$waif_pid $playlist
		set shuffled [playlist_shuffle $playlist]
		playlist_write ~/.waif/playqueue $shuffled
	}
	# waif_play will increment this to 0 to position at the head of the queue
	waif_write ~/.waif/current -1
	waif_write ~/.waif/next 0
	if [catch { exec kill -TERM $mpg123_pid } err] {
		log_error "$err"
	}
}

proc waif_reshuffle { } {
	if ![catch { glob ~/.waif/unshuffle.* } unshuffles] {
		foreach unshuffle $unshuffles {
			catch { file delete -force $unshuffle }
		}
	}
	waif_shuffle
}
	
proc waif_vol { args } {

	set procName waif_vol
	if [catch { eval exec aumix -v $args } err] { 
		log_error "<$args>: $err"
	} else {
		if { "$err" != "" } { puts "$err" }
	}
}

proc waif_wvol { args } {

	set procName waif_wvol
	if [catch { eval exec aumix -w $args } err] { 
		log_error "<$args>: $err"
	} else {
		if { "$err" != "" } { puts "$err" }
	}
}

proc waif_add { args } {
 	foreach a $args {
		playlist_add ~/.waif/playqueue $a
	}
}

proc waif_insert { num args } {
	foreach a [lsort -decreasing $args] {
		playlist_insert ~/.waif/playqueue $num $a
	}
}

proc waif_del { args } {
	playlist_del ~/.waif/playqueue $args
}

proc waif_save { file } {
	set procName waif_save
	if [catch { file copy -force ~/.waif/playqueue $file } err] {
		error "$err"
	}
}

proc waif_clear { { file ~/.waif/playqueue } } {
	if [file exists $file] {
		catch { file delete $file }
	}
}

proc waif_search { search } {
	set procName waif_search

	set result [playlist_search ~/.waif/playqueue "$search"]
	if { "$result" == "" } {
		return
	}
	foreach track $result {
		puts $track
	}
}

proc waif_create { file dir } {
	set procName waif_create

	Waif::Playlist::playlist_create $file $dir
}

proc waif_version { } {
	version
}

proc waif_help { } {
	help
}
 
} ;# end of namespace
