namespace eval Waif::Configure {
	global waifInfo Waif_version

proc configure_init { {file ~/.waifrc} } {
	if ![file exists $file] {
		puts "no waifrc found: creating $file"
		Waif::Configure::configure_create $file
	} else {
		Waif::Configure::configure_read $file
	}
}

proc configure_create { {file ~/.waifrc}  } {
	global waifInfo
	global Waif_version

	set procName configure_create
	# just a macro for readability
	set overwrite 1
	if [catch { open $file w } f] {
		log_error $f
		return
	} 
		
	# general
	configure_set version $Waif_version $overwrite
	configure_set url http://www.eds.org/~straycat/waif.html

	# loggging
	configure_set errorlog ~/.waif/error.log $overwrite

	# buffering 
	configure_set buff_delay 400

	# these options are deprecated
	configure_set buff_size "" $overwrite
	configure_set ogg_dvc "" $overwrite
	
	# cddb - on by default
	configure_set cddb 1

	# default show format
	configure_set defaultshow +1

	# write the config
	foreach name [lsort -dictionary [array names waifInfo]] {
		puts $f "$name=$waifInfo($name)"
	}
	close $f
}

proc configure_read { {file ~/.waifrc} } {
	global waifInfo Waif_version

	set procName configure_read
	if [catch { open $file r } f] {
		log_error $f
		return
	}
	while { [gets $f line] > -1 } {
		if ![regexp "(.*)=(.*)" $line all name value] {
			log_error "error in $file : \"$all\""
			return
		} else {
			set waifInfo($name) $value
		}
	}
	close $f
	if { [info exists waifInfo(version)] && \
		("$waifInfo(version)" != "$Waif_version") } {
		puts "version mismatch in waifrc - updating waifrc"
		configure_create $file
	}
}

proc configure_set { name value {overwrite 0} } {
	global waifInfo

	if { (![info exists waifInfo($name)]) || ($overwrite) } {
		set waifInfo($name) $value
	}
	if { "$value" == "" } {
		catch { unset waifInfo($name) }
	}
	return $value
}

proc configure_write { {file ~/.waifrc} } {
	global waifInfo

	if [catch { open $file w } f] {
		log_error $f
		return
	}
	if ![info exists waifInfo] {
		configure_create $file
		return
	}
	foreach name [lsort -dictionary [array names waifInfo]] {
		puts $f "$name=$waifInfo($name)"
	}
	close $f
}

proc configure_get { name } {
	global waifInfo

	if [info exists waifInfo($name)] {
		return $waifInfo($name)
	}
	return
}

} ;# end of Waif::Configure
