namespace eval ::Waif::Mime {
	global mimeInfo

	namespace export mime_get mime_select

proc mime_init { {file ~/.waif/mime} } {
	if ![file exists $file] {
		puts "no mime file found : creating $file"
		Waif::Mime::mime_create $file
	} else {
		Waif::Mime::mime_read $file
	}
}

proc mime_select { mp3 } {
	set mime [file extension $mp3]
	if { "$mime" != "" } {
		set cmd [mime_get $mime]
	} else {
		# no suffix was found, see if we recognize the prefix
		regexp (\[a-z]\[a-z]*):// $mp3 mime
		set cmd [mime_get $mime]
		# strip off the prefix
		if { "$mime" == "cda://" } {
			set mp3 [file tail $mp3]
		}
	}
	if { "$cmd" == "" } {
		set cmd "/bin/true %f"
		#waif_del $current
	}
	set idx [lsearch $cmd %f]
	if { $idx > 0 } {
		set cmd [lreplace $cmd $idx $idx $mp3]
	}
	if { "[Waif::waif_which [lindex $cmd 0]]" == "" } {
		set cmd [lreplace $cmd 0 0 /bin/true]
		#waif_del $current
	}
	return $cmd
}

proc mime_create { {file ~/.waif/mime} } {
	global mimeInfo

	set procName mime_create
	if [catch { open $file w } f] {
		log_error $f
		return
	}

	# mp3
	mime_set .mp3 "mpg123 -b 1024 %f"

	# ogg
	mime_set .ogg "ogg123 -d oss %f"

	# wav
	mime_set .wav "wavplay %f"

	# midi
	mime_set .midi "playmidi %f"

	# text-mode RealPlayer
	mime_set .ram "trplayer %f"

	# cdplayer
	mime_set cda:// "takcd %f"

	# icecast
	mime_set http:// "mpg123 %f"

	# write the config
	foreach name [lsort -dictionary [array names mimeInfo]] {
		puts $f "$name=$mimeInfo($name)"
	}
	mime_list
	close $f
}

proc mime_read { {file ~/.waif/mime} } {
	global mimeInfo

	set procName mime_read
	if [catch { open $file r } f] {
		log_error $f
		return
	}
	while { [gets $f line] > -1 } {
		if ![regexp "(.*)=(.*)" $line all name value] {
			log_error "error in $file : \"$all\""
			return
		} else {
			set mimeInfo($name) $value
		}
	}
	mime_list
	close $f
}
		
proc mime_write { {file ~/.waif/mime} } {
	global mimeInfo

	set procName mime_write
	if [catch { open $file w } f] {
		log_error $f
		return
	}
	if ![info exists mimeInfo] {
		mime_create $file
		return
	}
	foreach name [lsort -dictionary [array names mimeInfo]] {
		puts $f "$name=$mimeInfo($name)"
	}
	close $f
}
	
proc mime_set { mime value } {
	global mimeInfo

	if { "$value" != "" } {
		set mimeInfo($mime) $value
		return $value
	} else {
		catch { unset mimeInfo($mime) }
	}
}

proc mime_get { mime } {
	global mimeInfo

	if [info exists mimeInfo($mime)] {
		return $mimeInfo($mime)
	}
	return
}

proc mime_list { } {
	global mimeInfo

	if ![info exists mimeInfo] {
		return
	}
	set mimelist ""
	foreach mime [array names mimeInfo] {
		if { [lsearch $mimelist $mime] < 0 } {
			lappend mimelist $mime
		}
	}
	if { "$mimelist" != "" } {
		mime_set all $mimelist
	} else {
		return $mimelist
	}
}
		
}	;# end of Waif::Mime
