namespace eval ::Waif::Playlist {
	namespace export playlist_read playlist_write playlist_create \
		playlist_shuffle playlist_add playlist_del playlist_insert \
		playlist_search
	namespace import ::Waif::Mime::mime_get

proc playlist_read_dir { var dir } {
	upvar $var files
	set curdir [pwd]
	cd $dir 
	set wd [pwd]
	set f [exec ls -1]
	while { "$f" != "" } {
		set i [string first \n $f]
		if { $i < 0 } {
			set file [string range $f 0 end]
		} else {
			incr i -1
			set file [string range $f 0 $i]
		}
		if { [lsearch [mime_get all] "[file extension $file]"] > -1 } {
			lappend files(files) $wd/$file
		}
		set f [string trimleft $f $file]
		set f [string trimleft $f \n]
		if [file isdirectory $file] {
			lappend dirs $file
		}
	}
	if [info exists dirs] {
		foreach d $dirs {
			catch { playlist_read_dir files $d }
		}
	}
	cd $curdir
}

proc playlist_create { file dir } {
	set procName playlist_create
	playlist_read_dir _playlist $dir
	if ![info exists _playlist(files)] {
		return
	}
	if [catch { open $file w } f] {
		log_error "<$file,$dir>: $f"
		return
	}
	foreach mp3 $_playlist(files) {
		puts $f "$mp3"
	}
	close $f
	if ![file size $file] {
		file delete -force $file
	}
}
		

proc playlist_write { file playlist } {
	set procName playlist_write
	if [catch { open $file w } f] {
		log_error "<$file> : $f"
		return 1
	}
	foreach mp3 $playlist {
	# i'm not sure if we need to check the mime type a second time around
	# since playlist_read should have checked this first
	set this_code_may_not_be_needed {
		if { [lsearch [mime_get all] "[file extension $mp3]"] > -1 } {
			puts $f $mp3
		} elseif [regexp ^cda:// $mp3] {
			puts $f $mp3
		}
	}
		puts $f $mp3
	}
	close $f
}

proc playlist_read { file } {
	set procName playlist_read
	set cwd [pwd]
	if { [regexp ^(http|rtsp):// $file] } { return $file }
	if { [regexp "^cda://(.*)" $file all track] } {
		if { "$track" == "" } { 
			return [playlist_cda_read $file]
		} else {
			return $file
		}
	}
	if ![file exists $file] {
		puts "no such file : $file"
		return
	}
	if { [lsearch [mime_get all] "[file extension $file]"] > -1 } {
		# if this is a relative path, prepend cwd
		if { [string first / $file] != 0 } {
			set file "$cwd/$file"
		}
		return [list $file]
	}
	if { [file exists $file] && [file isdirectory $file] } {
		playlist_read_dir _playlist $file
		if ![info exists _playlist(files)] { return }
		return $_playlist(files)
	}
	#if { [regexp ^http:// $file] } { return $file }
	if [catch { open $file r } f] {
		log_error "<$file> : $f"
		return 
	}
	while { [gets $f line] >= 0 } {
		if { [lsearch [mime_get all] "[file extension $line]"] > -1 } {
			lappend playlist $line
		} else { 
			set tmp ""
			regexp (\[a-z]\[a-z]*):// $line tmp
			if { "[mime_get $tmp]" != "" } {
				lappend playlist $line
			}
		}
	}
	close $f
	if ![info exists playlist] { return }
	return $playlist
}

proc playlist_cda_read { file } {
	set procName playlist_cda_read
	set cda_app [lindex [Waif::Mime::mime_get cda://] 0]
	if [catch { exec -- $cda_app -d } discinfo] {
		log_error "<$file> $discinfo"
		return
	}
	set num_tracks [lindex $discinfo 1]
	set do_cddb [Waif::Configure::configure_get cddb]
	if { "$do_cddb" == "1" } {
		cddb_info mycd
	}
	for  { set i 1 } { $i <= $num_tracks } { incr i } {
		if [info exists mycd] {
			lappend playlist "cda://$mycd(artist)/$mycd($i)/$i"
		} else {
			lappend playlist cda://$i
		}
	}
	return $playlist
}

proc playlist_add { file playfile } {
	set playlist [playlist_read $file]
	foreach mp3 [playlist_read $playfile] {
		lappend playlist "$mp3"
	}
	playlist_write $file $playlist
}

proc playlist_del { file dels } {
	set procName playlist_del
	set playlist [playlist_read $file]
	foreach d $dels {
		if ![scan $d %d i] { 
			log_error "<$file,$d> argument should be integer"
		} else {
			# mark for deletion first
			set playlist [lreplace $playlist $i $i "<DELETE_ME>"]
		}
	}
	regsub "<DELETE_ME>" $playlist "" playlist
	playlist_write $file $playlist
}

proc playlist_insert { file index playfile } {
	set procName playlist_insert
	if ![scan $index %d i] {
		log_error "<$file,$index> argument should be integer"
		return
	}
	set playlist [playlist_read $file]
	foreach mp3 [lsort -decreasing [playlist_read $playfile]] {
		set playlist [linsert $playlist $i $mp3]
	}
	playlist_write $file $playlist
}

proc playlist_shuffle { playlist } {
	set procName playlist_shuffle
	# Shuffle a playlist (see rand.tcl to see how the list order is created).
	# Use rand_list to create a shuffled playorder and write that to 
	# playqueue.
	
	set playlist [playlist_read ~/.waif/playqueue]
	set playorder [rand_list [llength $playlist]]
	foreach mp3 $playorder {
		lappend shuffled [lindex $playlist $mp3]
	}
	return $shuffled
}

proc playlist_search { playlist search } {
	set procName playlist_seach

	set result {}
	set index 0
	set playlist [playlist_read ~/.waif/playqueue]
	foreach mp3 $playlist {
		set mp3 [Waif::waif_show_file $index [llength $playlist] $mp3]
		if { [regexp -nocase -- "$search" $mp3] } {
			lappend result $mp3
		}
		incr index
	}
	return $result
} 

} ;# end of namespace
